﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  [StorableClass]
  public class DoubleValueRange : Range<DoubleValue> {

    public override DoubleValue LowerBound {
      get {
        return base.LowerBound;
      }
      set {
        base.LowerBound = value;
      }
    }
    
    public DoubleValueRange(DoubleValue lowerBound, DoubleValue upperBound, DoubleValue stepSize) : base(lowerBound, upperBound, stepSize) { }
    public DoubleValueRange() { }
    [StorableConstructor]
    protected DoubleValueRange(bool deserializing) : base(deserializing) { }
    protected DoubleValueRange(DoubleValueRange original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DoubleValueRange(this, cloner);
    }

    protected override DoubleValue GetRandomSample(IRandom random) {
      double val;
      do {
        val = Math.Round((random.NextDouble() * (UpperBound.Value - LowerBound.Value) + LowerBound.Value) / StepSize.Value, 0) * StepSize.Value;
      } while (!IsInRange(val));
      return new DoubleValue(val);
    }

    public double ApplyStepSize(double value) {
      return (Math.Round(value / this.StepSize.Value, 0)) * this.StepSize.Value;
    }

    public bool IsInRange(double value) {
      return value <= this.UpperBound.Value && value >= this.LowerBound.Value;
    }

    public override IEnumerable<DoubleValue> GetCombinations() {
      var solutions = new List<DoubleValue>();
      double value = LowerBound.Value;

      while (value <= UpperBound.Value) {
        solutions.Add(new DoubleValue(value));
        value += StepSize.Value;
        value = ApplyStepSize(value);
      }
      return solutions;
    }

    protected override double CalculateSimilarityValue(DoubleValue a, DoubleValue b) {
      double range = UpperBound.Value - LowerBound.Value;
      double diff = Math.Abs(a.Value - b.Value);
      return Math.Max(0, (range - (diff*2)) / range);
    }
  }
}
